### =========================================================================== 
###
### Build coordnates mask
###
### ===========================================================================
# creates file coordinates_mask.nc used by mosaix from NEMO netcdf files 
# coordinates.nc and bathymetry.nc (on the same grid as coordinates.nc)

import numpy  as np
import xarray as xr
import nemo
import datetime, os, platform

# reproduce periodicity lbclnk bug for U and nperio = 4
nemoUbug = False
# change metrics (and bathymetry) for straits
straits = False
# periodicity imposed on coordinates, metrics and areas
coordperio = False
# function used to compute mask_T from bathymetry
maskbathynemo = False

# type of grid treated : ORCA 1, 2, paleorca # to be improved

#model   = 'eORCA1.2'
#model   = 'ORCA2.3'
#model   = 'ORCA2.4'
model   = 'paleORCA'
#model   = 'ORCA.05'

if model == 'eORCA1.2' :
    n_coord = 'eORCA1.2coordinates.nc' #'eORCA_R1_coordinates_v1.0.nc' 
    n_bathy = 'eORCA1.2bathy_meter.nc' #'eORCA_R1_bathy_meter_v2.2.nc'
    #n_coord = 'https://vesg.ipsl.upmc.fr/thredds/dodsC/work/p86mart/GRAF/DATA/eORCA_R1_coordinates_v1.0.nc'
    #n_bathy = 'https://vesg.ipsl.upmc.fr/thredds/dodsC/work/p86mart/GRAF/DATA/eORCA_R1_bathy_meter_v2.2.nc'
    nperio  = 6
    n_out   = 'eORCA1.2_coordinates_mask_test.nc'

if model == 'ORCA2.3' :
    nperio  = 4
    n_coord = 'coordinates_ORCA2.nc' # 'ORCA2.3_coordinates.nc' # 'coordinates_ORCA2_LIM3_PISCES.nc' #
    n_bathy = 'bathy_meter_ORCA2.nc' # 'ORCA2.3_bathy_meter.nc' # 'bathy_meter_ORCA2_LIM3_PISCES.nc' #
    n_out   = 'My_coordinates_xios_ORCA2.nc'
    
if model == 'ORCA2.4' :
    nperio  = 4
    n_coord = 'ORCA2.4_coordinates.nc'
    n_bathy = 'ORCA2.4_bathy_meter.nc'
    n_out   = 'ORCA2.4_coordinates_mask_test.nc'

if model == 'paleORCA' :
    n_coord = 'coordinates_paleo.nc'
    n_bathy = 'bathy_meter_paleo.nc'
    nperio = 6
    n_out = 'paleo_coordinates_mask_test.nc'

#---------------------------------------------------------------
# function to generate grid bounds from NEMO coordinates.nc file
def set_bounds (cdgrd) :
    '''
    Constructs lon/lat bounds
    Bounds are numerated counter clockwise, from bottom left
    See NEMO file OPA_SRC/IOM/iom.F90, ROUTINE set_grid_bounds, for more details
    '''
    # Define offset of coordinate representing bottom-left corner
    if cdgrd in ['T', 'W'] : 
        icnr = -1 ; jcnr = -1
        corner_lon = f_coord ['glamf'].copy() ; corner_lat = f_coord ['gphif'].copy()
        center_lon = f_coord ['glamt'].copy() ; center_lat = f_coord ['gphit'].copy()
    if cdgrd == 'U'        : 
        icnr =  0 ; jcnr = -1
        corner_lon = f_coord ['glamv'].copy() ; corner_lat = f_coord ['gphiv'].copy()
        center_lon = f_coord ['glamu'].copy() ; center_lat = f_coord ['gphiu'].copy()
    if cdgrd == 'V'        : 
        icnr = -1 ; jcnr =  0
        corner_lon = f_coord ['glamu'].copy() ; corner_lat = f_coord ['gphiu'].copy()
        center_lon = f_coord ['glamv'].copy() ; center_lat = f_coord ['gphiv'].copy()
    if cdgrd == 'F'        : 
        icnr = -1 ; jcnr = -1
        corner_lon = f_coord ['glamt'].copy() ; corner_lat = f_coord ['gphit'].copy()
        center_lon = f_coord ['glamf'].copy() ; center_lat = f_coord ['gphif'].copy()
      
    #y_grid, x_grid = corner_lon.shape ;
    nvertex = 4
    dims = ['y_grid_' + cdgrd, 'x_grid_' + cdgrd, 'nvertex_grid_' + cdgrd]
    
    bounds_lon = xr.DataArray (np.zeros ((jpj, jpi, nvertex)), dims=dims)
    bounds_lat = xr.DataArray (np.zeros ((jpj, jpi, nvertex)), dims=dims)
    
    idx = [(jcnr,icnr), (jcnr,icnr+1), (jcnr+1,icnr+1), (jcnr+1,icnr)]
    
    # Compute cell vertices that can be defined, 
    # and complete with periodicity
    for nn in range (nvertex) :
        tmp = np.roll (corner_lon, shift=tuple(-1*np.array(idx[nn])), axis=(-2,-1))
        bounds_lon[1:jpj,1:jpi,nn] = tmp[1:jpj,1:jpi]
        tmp = np.roll (corner_lat, shift=tuple(-1*np.array(idx[nn])), axis=(-2,-1))
        bounds_lat[1:jpj,1:jpi,nn] = tmp[1:jpj,1:jpi]
        bounds_lon[:,:,nn] = nemo.lbc (bounds_lon[:,:,nn], nperio=nperio, cd_type=cdgrd, nemo_4U_bug=nemoUbug)
        bounds_lat[:,:,nn] = nemo.lbc (bounds_lat[:,:,nn], nperio=nperio, cd_type=cdgrd, nemo_4U_bug=nemoUbug)
    
    # Zero-size cells at closed boundaries if cell points provided,
    # otherwise they are closed cells with unrealistic bounds
    if not (nperio == 1 or nperio == 4 or nperio == 6) :
        for nn in range (nvertex) : 
            bounds_lon[:,0,nn]   = center_lon[:,0]   # (West or jpni = 1), closed E-W
            bounds_lat[:,0,nn]   = center_lat[:,0]
            bounds_lon[:,jpi,nn] = center_lon[:,jpi] # (East or jpni = 1), closed E-W
            bounds_lat[:,jpi,nn] = center_lat[:,jpi]
    if nperio != 2 :
        for nn in range (nvertex) :
            bounds_lon[0,:,nn] = center_lon[0,:] # (South or jpnj = 1), not symmetric
            bounds_lat[0,:,nn] = center_lat[0,:]
    if nperio < 3 :
        for nn in range (nvertex) :
            bounds_lon[jpj,:,nn] = center_lon[jpj,:] # (North or jpnj = 1), no north fold
            bounds_lat[jpj,:,nn] = center_lat[jpj,:]
    
    # Rotate cells at the north fold
    if nperio >= 3 :
        # Working array for location of northfold
        z_fld = nemo.lbc (np.ones ((jpj, jpi)), nperio=nperio, cd_type=cdgrd, psgn=-1., nemo_4U_bug=nemoUbug)
        z_fld = np.repeat((z_fld == -1.0)[...,np.newaxis],4,axis=2)
        # circular shift of 2 indices in bounds third index
        bounds_lon_tmp = np.roll (bounds_lon, shift=-2, axis=2)
        bounds_lat_tmp = np.roll (bounds_lat, shift=-2, axis=2)
        bounds_lon[:,:,:]  = np.where (z_fld, bounds_lon_tmp[:,:,:] , bounds_lon[:,:,:] )
        bounds_lat[:,:,:]  = np.where (z_fld, bounds_lat_tmp[:,:,:] , bounds_lat[:,:,:] )
    
    # Invert cells at the symmetric equator
    if nperio == 2 :
        bounds_lon_tmp = np.roll (bounds_lon, shift=-2, axis=2)
        bounds_lat_tmp = np.roll (bounds_lat, shift=-2, axis=2)
        bounds_lon [0,:,:] = bounds_lon [0,:,:]
        bounds_lat [0,:,:] = bounds_lat [0,:,:]
    
    #bounds_lon.attrs['coordinates'] = 'nav_lat_grid_' + cdgrd + ' nav_lon_grid_' + cdgrd
    #bounds_lat.attrs['coordinates'] = 'nav_lat_grid_' + cdgrd + ' nav_lon_grid_' + cdgrd
    bounds_lon.attrs['units']       = 'degrees_east'
    bounds_lat.attrs['units']       = 'degrees_north'
    bounds_lon.name = 'bounds_lon_grid_' + cdgrd
    bounds_lat.name = 'bounds_lat_grid_' + cdgrd
    # remove _FillValue
    bounds_lon.encoding['_FillValue'] = None
    bounds_lat.encoding['_FillValue'] = None

    return bounds_lon, bounds_lat
#------------------------------------------------------------

# open input files while removing the time dimension
f_coord = xr.open_dataset (n_coord, decode_times=False).squeeze()
f_bathy = xr.open_dataset (n_bathy, decode_times=False).squeeze()

# Suppress time if necessary
try    :
    del f_coord['time']
    print ('time successfully removed')
except :
    pass
    print ('failed to suppress time')

Bathymetry = f_bathy['Bathymetry'].copy()

try :
    Bathymetry = Bathymetry.rename ({'nav_lat':'nav_lat_grid_T', 'nav_lon':'nav_lon_grid_T'})
    print ('Bathymetry file: Normal case')
except : 
    print ('Bathymetry file: Exception')
    nav_lon_grid_T = f_bathy ['nav_lon']
    nav_lat_grid_T = f_bathy ['nav_lat']
    Bathymetry = xr.DataArray (Bathymetry, coords = { "nav_lat_grid_T": (["y", "x"], nav_lat_grid_T),
                                                      "nav_lon_grid_T": (["y", "x"], nav_lon_grid_T) } )
    
Bathymetry = Bathymetry.rename ({'y':'y_grid_T', 'x':'x_grid_T'})

if straits :
    # Open straits for usual grids
    if model == 'ORCA2.3' :
        # orca_r2: Gibraltar strait open
        Bathymetry[101,139] = 284.
        # orca_r2: Bab el Mandeb strait open
        Bathymetry[87,159] = 137.

# Determine periodicity from grid type
jpj, jpi = Bathymetry.shape
try : 
    if nperio != None :
        print ('nperio specified : ', nperio)
except :
    nperio = None
    if jpi ==  182 : nperio = 4 # ORCA2. \!/ We choose legacy orca2
    if jpi ==  362 : nperio = 6 # ORCA1.
    if jpi == 1442 : nperio = 6 # ORCA025.
    #
    if nperio == None :
        sys.exit ('nperio not found, and cannot by guessed' )
    else :
        print ('nperio set as {:d} (deduced from data size {:d} {:d})'.format (nperio, jpj, jpi))

Bathymetry = nemo.lbc (Bathymetry, nperio=nperio, cd_type='T')
#Suppress ocean points at southernmost position
Bathymetry[0,:] = 0.0 

# Creation of mask_T

if maskbathynemo :
    # Use same formula as domzgr. 
    mask_T = xr.where (Bathymetry - 1. + 0.1 >= 0.0, 1, 0).astype (dtype='int8')
else :
    mask_T = xr.where (Bathymetry > 0.0, 1, 0).astype (dtype='int8')

# Creation of U, V, W, F masks from mask_T
mask_U = mask_T * mask_T.shift (x_grid_T=-1)
mask_V = mask_T * mask_T.shift (y_grid_T=-1)
mask_F = mask_T * mask_T.shift (y_grid_T=-1) * mask_T.shift (x_grid_T=-1) * mask_T.shift (y_grid_T=-1, x_grid_T=-1)
mask_W = mask_T

mask_U = nemo.lbc (mask_U, nperio=nperio, cd_type='U', nemo_4U_bug=nemoUbug).astype (dtype='int8')
mask_V = nemo.lbc (mask_V, nperio=nperio, cd_type='V').astype (dtype='int8')
mask_F = nemo.lbc (mask_F, nperio=nperio, cd_type='F').astype (dtype='int8')
mask_W = nemo.lbc (mask_W, nperio=nperio, cd_type='W').astype (dtype='int8')

mask_U = mask_U.rename ( {'y_grid_T'      : 'y_grid_U'      , 'x_grid_T'      : 'x_grid_U', 
                          'nav_lat_grid_T': 'nav_lat_grid_U', 'nav_lon_grid_T': 'nav_lon_grid_U'} )
mask_V = mask_V.rename ( {'y_grid_T'      : 'y_grid_V'      , 'x_grid_T'      : 'x_grid_V',
                          'nav_lat_grid_T': 'nav_lat_grid_V', 'nav_lon_grid_T': 'nav_lon_grid_V'} )
mask_W = mask_W.rename ( {'y_grid_T'      : 'y_grid_W'      , 'x_grid_T'      : 'x_grid_W',
                          'nav_lat_grid_T': 'nav_lat_grid_W', 'nav_lon_grid_T': 'nav_lon_grid_W'} )
mask_F = mask_F.rename ( {'y_grid_T'      : 'y_grid_F'      , 'x_grid_T'      : 'x_grid_F',
                          'nav_lat_grid_T': 'nav_lat_grid_F', 'nav_lon_grid_T': 'nav_lon_grid_F'} )

mask_T.name = 'mask_T'
mask_U.name = 'mask_U'
mask_V.name = 'mask_V'
mask_F.name = 'mask_F'
mask_W.name = 'mask_W'

# create masks without duplicate points : maskutil_[TUVWF]
for cd_type in ['T', 'U', 'V', 'F', 'W'] :
    MaskName = 'mask_'     + cd_type
    UtilName = 'maskutil_' + cd_type
    locals()[UtilName] = nemo.lbc_mask (locals()[MaskName].copy(), nperio=nperio, cd_type=cd_type)
    locals()[UtilName].name = UtilName

# add masks attributes
mask_T.attrs    ['cell_measures'] = 'area: area_grid_T'
mask_U.attrs    ['cell_measures'] = 'area: area_grid_U'
mask_V.attrs    ['cell_measures'] = 'area: area_grid_V'
mask_W.attrs    ['cell_measures'] = 'area: area_grid_W'
mask_F.attrs    ['cell_measures'] = 'area: area_grid_F'

maskutil_T.attrs['cell_measures'] = 'area: area_grid_T'
maskutil_U.attrs['cell_measures'] = 'area: area_grid_U'
maskutil_V.attrs['cell_measures'] = 'area: area_grid_V'
maskutil_W.attrs['cell_measures'] = 'area: area_grid_W'
maskutil_F.attrs['cell_measures'] = 'area: area_grid_F'

# create grid coordinates from NEMO coordinates.nc file
if coordperio :
    nav_lon_grid_T = nemo.lbc (f_coord ['glamt'].copy(), nperio=nperio, cd_type='T')
    nav_lat_grid_T = nemo.lbc (f_coord ['gphit'].copy(), nperio=nperio, cd_type='T')
    nav_lon_grid_U = nemo.lbc (f_coord ['glamu'].copy(), nperio=nperio, cd_type='U',nemo_4U_bug=nemoUbug)
    nav_lat_grid_U = nemo.lbc (f_coord ['gphiu'].copy(), nperio=nperio, cd_type='U',nemo_4U_bug=nemoUbug)
    nav_lon_grid_V = nemo.lbc (f_coord ['glamv'].copy(), nperio=nperio, cd_type='V')
    nav_lat_grid_V = nemo.lbc (f_coord ['gphiv'].copy(), nperio=nperio, cd_type='V')
    nav_lon_grid_W = nemo.lbc (f_coord ['glamt'].copy(), nperio=nperio, cd_type='W')
    nav_lat_grid_W = nemo.lbc (f_coord ['gphit'].copy(), nperio=nperio, cd_type='W')
    nav_lon_grid_F = nemo.lbc (f_coord ['glamf'].copy(), nperio=nperio, cd_type='F')
    nav_lat_grid_F = nemo.lbc (f_coord ['gphif'].copy(), nperio=nperio, cd_type='F')
else :
    nav_lon_grid_T = f_coord ['glamt'].copy()
    nav_lat_grid_T = f_coord ['gphit'].copy()
    nav_lon_grid_U = f_coord ['glamu'].copy()
    nav_lat_grid_U = f_coord ['gphiu'].copy()
    nav_lon_grid_V = f_coord ['glamv'].copy()
    nav_lat_grid_V = f_coord ['gphiv'].copy()
    nav_lon_grid_W = f_coord ['glamt'].copy()
    nav_lat_grid_W = f_coord ['gphit'].copy()
    nav_lon_grid_F = f_coord ['glamf'].copy()
    nav_lat_grid_F = f_coord ['gphif'].copy()

nav_lon_grid_T = nav_lon_grid_T.rename( {'y':'y_grid_T', 'x':'x_grid_T'} )
nav_lat_grid_T = nav_lat_grid_T.rename( {'y':'y_grid_T', 'x':'x_grid_T'} )
nav_lon_grid_U = nav_lon_grid_U.rename( {'y':'y_grid_U', 'x':'x_grid_U'} )
nav_lat_grid_U = nav_lat_grid_U.rename( {'y':'y_grid_U', 'x':'x_grid_U'} )
nav_lon_grid_V = nav_lon_grid_V.rename( {'y':'y_grid_V', 'x':'x_grid_V'} )
nav_lat_grid_V = nav_lat_grid_V.rename( {'y':'y_grid_V', 'x':'x_grid_V'} )
nav_lon_grid_W = nav_lon_grid_W.rename( {'y':'y_grid_W', 'x':'x_grid_W'} )
nav_lat_grid_W = nav_lat_grid_W.rename( {'y':'y_grid_W', 'x':'x_grid_W'} )
nav_lon_grid_F = nav_lon_grid_F.rename( {'y':'y_grid_F', 'x':'x_grid_F'} )
nav_lat_grid_F = nav_lat_grid_F.rename( {'y':'y_grid_F', 'x':'x_grid_F'} )

# remove _FillValue and missing_value
for cd_type in ['T', 'U', 'V', 'F', 'W'] :
    for dir_type in ['lat', 'lon'] :
        coord_name = 'nav_' + dir_type + '_grid_' + cd_type
        locals()[coord_name].encoding['_FillValue'] = None
        locals()[coord_name].encoding['missing_value'] = None

nav_lon_grid_T.name = 'nav_lon_grid_T'
nav_lat_grid_T.name = 'nav_lat_grid_T'
nav_lon_grid_U.name = 'nav_lon_grid_U'
nav_lat_grid_U.name = 'nav_lat_grid_U'
nav_lon_grid_V.name = 'nav_lon_grid_V'
nav_lat_grid_V.name = 'nav_lat_grid_V'
nav_lon_grid_F.name = 'nav_lon_grid_F'
nav_lat_grid_F.name = 'nav_lat_grid_F'
nav_lon_grid_W.name = 'nav_lon_grid_W'
nav_lat_grid_W.name = 'nav_lat_grid_W'

# add coordinates attributes
nav_lon_grid_T.attrs['standard_name'] = 'longitude'
nav_lon_grid_T.attrs['long_name']     = 'Longitude'
nav_lon_grid_T.attrs['units']         = 'degrees_east'
nav_lat_grid_T.attrs['standard_name'] = 'latitude'
nav_lat_grid_T.attrs['long_name']     = 'Latitude'
nav_lat_grid_T.attrs['units']         = 'degrees_north'

nav_lon_grid_U.attrs['standard_name'] = 'longitude'
nav_lon_grid_U.attrs['long_name']     = 'Longitude'
nav_lon_grid_U.attrs['units']         = 'degrees_east'
nav_lat_grid_U.attrs['standard_name'] = 'latitude'
nav_lat_grid_U.attrs['long_name']     = 'Latitude'
nav_lat_grid_U.attrs['units']         = 'degrees_north'

nav_lon_grid_V.attrs['standard_name'] = 'longitude'
nav_lon_grid_V.attrs['long_name']     = 'Longitude'
nav_lon_grid_V.attrs['units']         = 'degrees_east'
nav_lat_grid_V.attrs['standard_name'] = 'latitude'
nav_lat_grid_V.attrs['long_name']     = 'Latitude'
nav_lat_grid_V.attrs['units']         = 'degrees_north'

nav_lon_grid_W.attrs['standard_name'] = 'longitude'
nav_lon_grid_W.attrs['long_name']     = 'Longitude'
nav_lon_grid_W.attrs['units']         = 'degrees_east'
nav_lat_grid_W.attrs['standard_name'] = 'latitude'
nav_lat_grid_W.attrs['long_name']     = 'Latitude'
nav_lat_grid_W.attrs['units']         = 'degrees_north'

nav_lon_grid_F.attrs['standard_name'] = 'longitude'
nav_lon_grid_F.attrs['long_name']     = 'Longitude'
nav_lon_grid_F.attrs['units']         = 'degrees_east'
nav_lat_grid_F.attrs['standard_name'] = 'latitude'
nav_lat_grid_F.attrs['long_name']     = 'Latitude'
nav_lat_grid_F.attrs['units']         = 'degrees_north'

nav_lon_grid_T.attrs['bounds'] = 'bounds_lon_grid_T'
nav_lat_grid_T.attrs['bounds'] = 'bounds_lat_grid_T'
nav_lon_grid_U.attrs['bounds'] = 'bounds_lon_grid_U'
nav_lat_grid_U.attrs['bounds'] = 'bounds_lat_grid_U'
nav_lon_grid_V.attrs['bounds'] = 'bounds_lon_grid_V'
nav_lat_grid_V.attrs['bounds'] = 'bounds_lat_grid_V'
nav_lon_grid_W.attrs['bounds'] = 'bounds_lon_grid_W'
nav_lat_grid_W.attrs['bounds'] = 'bounds_lat_grid_W'
nav_lon_grid_F.attrs['bounds'] = 'bounds_lon_grid_F'
nav_lat_grid_F.attrs['bounds'] = 'bounds_lat_grid_F'

# create areas variables for grid cells from NEMO coordinates.nc file
# create new variables e1 e2 to keep f_coord the same
for cd_type in ['t', 'u', 'v', 'f'] :
    for axis in ['1', '2'] :
        coordName = 'e' + axis + cd_type
        locals()[coordName]=f_coord[coordName].copy()
        # remove zero values from areas
        # need to be define for the extended grid south of -80S
        # some point are undefined but you need to have e1 and e2 .NE. 0
        locals()[coordName]=xr.where(locals()[coordName] == 0.0, 1.0e2, locals()[coordName])

#Compute cells areas

# Correct areas for straits
if straits :
    # ORCA R2 configuration
    if model == 'ORCA2.3' :
        # Gibraltar     : e2u reduced to 20 km
        e2u[101,138:140] = 20.e3
        # Bab el Mandeb : e2u reduced to 30 km
        #                e1v reduced to 18 km
        e1v[87,159] = 18.e3
        e2u[87,159] = 30.e3
        # Danish Straits: e2u reduced to 10 km
        e2u[115,144:146] = 10.e3
    # ORCA R1 configuration
    if model == 'eORCA1.2' :
        # Gibraltar : e2u reduced to 20 km
        e2u[240,281:283] = 20.e3
        # Bhosporus : e2u reduced to 10 km
        e2u[247,313:315] =  10.e3
        # Lombok : e1v reduced to 13 km
        e1v[163:165,43] =  13.e3
        # Sumba : e1v reduced to 8 km
        e1v[163:165,47] =  8.e3
        # Ombai : e1v reduced to 13 km
        e1v[163:165,52] = 13.e3
        # Timor Passage : e1v reduced to 20 km
        #e1v[163:165,55] = 20.e3
        # W Halmahera : e1v reduced to 30 km
        e1v[180:182,54] = 30.e3
        # E Halmahera : e1v reduced to 50 km
        e1v[180:182,57] = 50.e3
    # ORCA R05 configuration
    if model == 'ORCA.05' :
        # Reduced e2u at the Gibraltar Strait
        e2u[326,562:564] =  20.e3
        # Reduced e2u at the Bosphore Strait
        e2u[342,626:628] =  10.e3
        # Reduced e2u at the Sumba Strait
        e2u[231,92:94] =  40.e3
        # Reduced e2u at the Ombai Strait
        e2u[231,102] =  15.e3
        # Reduced e2u at the Palk Strait
        e2u[269,14] =  10.e3
        # Reduced e1v at the Lombok Strait
        e1v[231:233,86] =  10.e3
        # Reduced e1v at the Bab el Mandeb
        e1v[275,661] =  25.e3

if coordperio :
    area_grid_T = nemo.lbc (e1t*e2t, nperio=nperio, cd_type='T')
    area_grid_U = nemo.lbc (e1u*e2u, nperio=nperio, cd_type='U',nemo_4U_bug=nemoUbug)
    area_grid_V = nemo.lbc (e1v*e2v, nperio=nperio, cd_type='V')
    area_grid_W = nemo.lbc (e1t*e2t, nperio=nperio, cd_type='W')
    area_grid_F = nemo.lbc (e1f*e2f, nperio=nperio, cd_type='F')
else :
    area_grid_T = e1t*e2t
    area_grid_U = e1u*e2u
    area_grid_V = e1v*e2v
    area_grid_W = e1t*e2t
    area_grid_F = e1f*e2f

area_grid_T.name = 'area_grid_T'
area_grid_U.name = 'area_grid_U'
area_grid_V.name = 'area_grid_V'
area_grid_F.name = 'area_grid_F'
area_grid_W.name = 'area_grid_W'

area_grid_T = area_grid_T.rename ({'y':'y_grid_T', 'x':'x_grid_T'})
area_grid_U = area_grid_U.rename ({'y':'y_grid_U', 'x':'x_grid_U'})
area_grid_V = area_grid_V.rename ({'y':'y_grid_V', 'x':'x_grid_V'})
area_grid_W = area_grid_W.rename ({'y':'y_grid_W', 'x':'x_grid_W'})
area_grid_F = area_grid_F.rename ({'y':'y_grid_F', 'x':'x_grid_F'})

area_grid_T.attrs['standard_name'] = 'cell_area'
area_grid_T.attrs['units']         = 'm2'
area_grid_U.attrs['standard_name'] = 'cell_area'
area_grid_U.attrs['units']         = 'm2'
area_grid_V.attrs['standard_name'] = 'cell_area'
area_grid_V.attrs['units']         = 'm2'
area_grid_W.attrs['standard_name'] = 'cell_area'
area_grid_W.attrs['units']         = 'm2'
area_grid_F.attrs['standard_name'] = 'cell_area'
area_grid_F.attrs['units']         = 'm2'

for cd_type in ['T', 'U', 'V', 'F', 'W'] :
    areaName = 'area_grid_' + cd_type
    locals()[areaName].encoding['_FillValue'] = None

# compute grid cells bounds
bounds_lon_grid_T, bounds_lat_grid_T = set_bounds ('T')
bounds_lon_grid_U, bounds_lat_grid_U = set_bounds ('U')
bounds_lon_grid_V, bounds_lat_grid_V = set_bounds ('V')
bounds_lon_grid_W, bounds_lat_grid_W = set_bounds ('W')
bounds_lon_grid_F, bounds_lat_grid_F = set_bounds ('F')

# build xarray dataset to be saved
ds = xr.Dataset ({
    'mask_T'      : mask_T     ,
    'mask_U'      : mask_U     ,
    'mask_V'      : mask_V     ,
    'mask_W'      : mask_W     ,
    'mask_F'      : mask_F     ,
    'area_grid_T' : area_grid_T,
    'area_grid_U' : area_grid_U,
    'area_grid_V' : area_grid_V,
    'area_grid_W' : area_grid_W,
    'area_grid_F' : area_grid_F,
    'maskutil_T'  : maskutil_T ,
    'maskutil_U'  : maskutil_U ,
    'maskutil_V'  : maskutil_V ,
    'maskutil_W'  : maskutil_W ,
    'maskutil_F'  : maskutil_F ,
    'bounds_lon_grid_T': bounds_lon_grid_T,
    'bounds_lat_grid_T': bounds_lat_grid_T,
    'bounds_lon_grid_U': bounds_lon_grid_U,
    'bounds_lat_grid_U': bounds_lat_grid_U,
    'bounds_lon_grid_V': bounds_lon_grid_V,
    'bounds_lat_grid_V': bounds_lat_grid_V,
    'bounds_lon_grid_W': bounds_lon_grid_W,
    'bounds_lat_grid_W': bounds_lat_grid_W,
    'bounds_lon_grid_F': bounds_lon_grid_F,
    'bounds_lat_grid_F': bounds_lat_grid_F,
})

#replace nav_lon nav_lat with variables obtained from NEMO coordinates.nc file
#by construction nav_lon nav_lat are those of the bathymetry
for cd_type in ['T', 'U', 'V', 'F', 'W'] :
    for dir_type in ['lat', 'lon'] :
        coord_name = 'nav_' + dir_type + '_grid_' + cd_type
        ds.coords[coord_name]=locals()[coord_name]

ds.attrs['name']         = 'coordinates_mask'
ds.attrs['description']  = 'coordinates and mask for MOSAIX'
ds.attrs['title']        = 'coordinates_mask'
ds.attrs['source']       = 'IPSL Earth system model'
ds.attrs['group']        = 'ICMC IPSL Climate Modelling Center'
ds.attrs['Institution']  = 'IPSL https.//www.ipsl.fr'
ds.attrs['Model']        = model
ds.attrs['timeStamp']    = '{:%Y-%b-%d %H:%M:%S}'.format (datetime.datetime.now ())
ds.attrs['history']      = 'Build from ' + n_coord + ' and ' + n_bathy
ds.attrs['directory']    = os.getcwd     ()
ds.attrs['user']         = os.getlogin   ()
ds.attrs['HOSTNAME']     = platform.node ()
ds.attrs['Python']       = 'Python version: ' +  platform.python_version ()
ds.attrs['xarray']       = 'xarray version: ' +  xr.__version__
ds.attrs['OS']           = platform.system  ()
ds.attrs['release']      = platform.release ()
ds.attrs['hardware']     = platform.machine ()

ds.attrs['SVN_Author']   = "$Author:  $"
ds.attrs['SVN_Date']     = "$Date:  $"
ds.attrs['SVN_Revision'] = "$Revision:  $"
ds.attrs['SVN_Id']       = "$Id: Build_coordinates_mask.py $"
ds.attrs['SVN_HeadURL']  = "$HeadURL: http://forge.ipsl.jussieu.fr/igcmg/svn/TOOLS/MOSAIX/Build_coordinates_mask.py $"

# save to output file
ds.to_netcdf (n_out)
